#!/usr/bin/perl -w
##########################################################################
# $Id: FaultInjector.pl,v 1.3 2004/03/23 03:41:06 cfry Exp $
#
# Description: A simple Fault Injector
# - Injects faults into the system into the distributed system
#   by killing replicated processes at the user specified rate
#
# Use:	FaultInjector.pl [-h] [-v] [-d <seconds>]
# Switches: -h : Display Help
#           -v : Use Verbose Mode to display activity
#           -d <seconds> : delay between process terminations
#
# Limitations/Assumptions:
# - User must have a working Replication Manager (repman)
# - User must launch processes via repman
##########################################################################
use strict;

my $verbose = 0;		# for debugging...
my @MYARGV = @ARGV;             # argument list, saved for later...

# Default configuration values
my $user = $ENV{'USER'};         # the username under which commands are run
my $hostname = $ENV{'HOSTNAME'}; # the local hostname
my $prefix = 'Replica_';
my $repman = './repman';	 # command to invoke ReplicationManager
my $delay  = 8;                  # delay between ping sessions

# Subroutine prototypes
sub FIHelp();          # Display usage
sub FIProcess();      # read RC file & process labels/commands

#-------------------------------------------------------------------------
# Main - the main application logic
main: {
  print "\nStarting Fault Injector...\n";

  # process the first argument to determine the mode
  # - once mode determined, call the appropriate subroutine
  while(my $arg = shift) {
    # repman -k <label>
    SWITCH: for($arg) {
    /-h/ && do {	# Show Help/Usage
      &FIHelp();             # - Show help...
      exit; };               #   ...and terminate
    /-v/ && do {	# Enable Verbose
      $verbose = 1;
      last SWITCH; };
    /-d/ && do {	# Specify delay
      $delay = shift or die "ERROR - Missing argument\n\t$!\n";
      last SWITCH; };
    print "ERROR - Invalid Switch:$arg\n";
    &FIHelp();
    }
  }

  # Begin monitoring process
  # - run in an endless loop, with a delay between interations
  # - Note: For FaultTolerance within FaultDetector do this:
  #   1) add initial FaultDetector process in RC file
  #   2) add a second FaultDetector process to RC file
  #   3) start initial FaultDetector via repman.
  #   Thus, FaultDetectors will monitor each other so that if
  #   one dies the other will restart it.
  if( $verbose ) {
    print "Verbose mode!\n" .
          "User: $user\@$hostname\n" .
          "Args[$#MYARGV]:@MYARGV\n" .
          "delay:$delay\n";
  }
  while(1) {
    &FIProcess();
    print "Sleeping for $delay seconds...\n";
    sleep $delay;            # sleep for n-seconds
  };
  print "Terminating...\n";
}

#-------------------------------------------------------------------------
# Displays usage syntax to user
sub FIHelp() {
  die "Syntax: FI.pl -h\n" .
      "        FI.pl [-v] [-d <seconds>]\n";
      "where:\n" .
      "  -h : display help\n" .
      "  -v : verbose mode\n" .
      "  -d <seconds> : delay between each termination\n";
}

#-------------------------------------------------------------------------
# Injects a fault
# - Queries "repman" for the list of running processes.
# - A process is randomly selected from the list and terminated.
sub FIProcess() {

  # Invoke Repman to obtain list of running processes
  print "\nQuerying RepMan...\n";
  my $cmdline = "$repman";
  print " $cmdline\n" if $verbose;
  my @repmanOutput = grep /$prefix/, `$cmdline`;
  my $lines = $#repmanOutput + 1;
  if ($verbose) {
    print " repmanOutput: -- Lines:$lines\n";
    foreach my $line (@repmanOutput) {
      print $line;
    }
  }

  # Randomly select a process to kill
  if ($lines > 0) {
    my $line = $repmanOutput[rand $lines];
    my ($label,$host) = split(" ",$line);
    print "Killing $label...\n";
    # $cmdline = "$repman -k $label";
    $cmdline = "ssh $host killall -9 java";
    print " $cmdline\n" if $verbose;
    system($cmdline);
   } else {
    print "-- no processes running - nothing to do... --";
  }

}


