// $Id: ManagerRemote.java,v 1.11 2004/03/02 17:21:55 pmwilson Exp $

package com.spamnbeans.server;

//import com.spamnbeans.server.*;

import java.rmi.RemoteException;
import javax.ejb.FinderException;
import javax.ejb.CreateException;
import javax.ejb.RemoveException;
import java.util.HashSet;

/**
 * Provide the external interface to the ManagerBean.
 * 
 * <p>The ManagerBean is responsible for creating, finding, and removing 
 * ClientBeans and ServerBeans, which are used mainly for validation and 
 * statistics gathering.
 *
 * @author Andrew Boyer,
 *         <a href="mailto:aboyer@andrew.cmu.edu">aboyer@andrew.cmu.edu</a>
 * @version 1.0
 */
public interface ManagerRemote extends javax.ejb.EJBObject {

    /**
     * Create a new Server EntityBean
     * @param serverID - The ServerID (primary key) of the created Bean
     * @param name - The name of the created Bean
     * @param isEnabled - The enabled status of server
     */
    public void serverCreate( Integer serverID, String name, boolean isEnabled)
	throws RemoteException, CreateException, DuplicateServerException;

    /**
     * Remove Server EntityBean
     * @param serverID The ServerID of the removed Bean
     */
    public void serverRemove( Integer serverID )
	throws RemoteException, InvalidServerException, RemoveException;

    /**
     * Check if a Server EntityBean already exists
     * @param serverID The ServerID of the Bean to check
     */
    public boolean serverExists( Integer serverID )
	throws RemoteException;

    /**
     * Get the Name of a Server EntityBean
     * @param serverID The ServerID of the Bean to check
     */
    public String serverGetName( Integer serverID )
	throws RemoteException, InvalidServerException;

    /**
     * Get the ServerID of a Server EntityBean
     * @param name The Name of the Bean to check
     */
    public Integer serverGetID( String name )
	throws RemoteException, InvalidServerException;

    /**
     * Get a list of all existing Server EntityBeans by ID
     */
    public HashSet serverListIDs()
	throws RemoteException, FinderException;

    /**
     * Get a list of all existing Server EntityBeans by name
     */
    public HashSet serverListNames()
	throws RemoteException, FinderException;

    /**
     * Check if a Server EntityBean is Enabled
     * @param name - The ServerID of the Bean to check
     */
    public boolean serverIsEnabled( String name )
	throws RemoteException, InvalidServerException;
    
    /**
     * Check if a Server EntityBean is Enabled
     * @param serverID - The ServerID of the Bean to check
     */
    public boolean serverIsEnabled( Integer serverID )
	throws RemoteException, InvalidServerException;
    
    /**
     * Enable or Disable a Server EntityBean
     * @param name The name of the Bean to change
     * @param isEnabled 
     */
    public void serverSetEnabled(String name, boolean Enable)
	throws RemoteException, InvalidServerException;
    public void serverSetEnabled(Integer serverID, boolean Enable)
	throws RemoteException, InvalidServerException;

    /**
     * Get a list of all Enabled Server EntityBeans
     */
    public HashSet serverListEnabled()
	throws RemoteException, FinderException;

    /**
     * Return the ID of one active Server EntityBean
     */
    public String serverLoadBalance()
	throws RemoteException, FinderException, InvalidServerException;



    /**
     * Create a new Client EntityBean
     * @param clientID The ClientID (primary key) of the created Bean
     */
    public void clientCreate( Integer clientID, String name )
	throws RemoteException, CreateException, DuplicateClientException;

    /**
     * Remove Client EntityBean
     * @param clientID The ClientID of the removed Bean
     */
    public void clientRemove( Integer clientID )
	throws RemoteException, InvalidClientException, RemoveException;

    /**
     * Check if a Client EntityBean already exists
     * @param clientID The ClientID of the Bean to check
     */
    public boolean clientExists( Integer clientID )
	throws RemoteException;

    /**
     * Get the Name of a Client EntityBean
     * @param clientID The ClientID of the Bean to check
     */
    public String clientGetName( Integer clientID )
	throws RemoteException, InvalidClientException;

    /**
     * Get the ClientID of a Client EntityBean
     * @param name The Name of the Bean to check
     */
    public Integer clientGetID( String name )
	throws RemoteException, InvalidClientException;

    /**
     * Get a list of all running Client EntityBeans
     */
    public HashSet clientListIDs()
	throws RemoteException, FinderException;
    
    /**
     * Get stats of Msg EntityBeans for a particular Client EntityBean
     * TODO: Update to return a class/struct with all stat info
     */
    public String msgGetStatsByClient(Integer clientID) 
	throws RemoteException, FinderException;

    /**
     * Get a list of all Msg EntityBeans
     */
    public HashSet msgListIDs()
	throws RemoteException, FinderException;

    /**
     * Remove Msg EntityBean
     * @param msgID The MsgID of the removed Bean
     */
    public void msgRemove( Integer msgID )
	throws RemoteException, InvalidMessageIDException, RemoveException;
}
