// $Id: ServerStatBean.java,v 1.6 2004/03/23 00:01:12 gca Exp $

package com.spamnbeans.server;

import java.util.logging.*;
 
/**
 * Define the ServerStatBean methods.
 * 
 * <p>
 * The ServerStat Bean tracks the statistics for a Server, identified by its
 * Name and ServerID.
 * 
 * @author Andrew Boyer, <a href="mailto:aboyer@andrew.cmu.edu">
 *         aboyer@andrew.cmu.edu</a>
 * @version 1.0
 */
public abstract class ServerStatBean implements javax.ejb.EntityBean {
	private javax.ejb.EntityContext ctx;
	private static Logger logger =
		Logger.getLogger(ServerStatBean.class.getName());

	/**
	 * Create a new ServerStat EntityBean
	 * 
	 * @param serverID -
	 *            The ID for the new Server
	 * @param name -
	 *            the name for the new Server
	 * @param isEnabled -
	 *            The new server's <I>enabled</I> state
	 */
	public Integer ejbCreate(Integer serverID, String name, boolean isEnabled)
		throws javax.ejb.CreateException {
		this.setServerID(serverID);
		this.setName(name);
		this.setEnable(isEnabled);
		return null;
	}

	public void ejbPostCreate(Integer serverID, String name, boolean isEnabled) {
		String strMsg = "ServerStatBean:ejbPostCreate(" 
			+ serverID
			+ "," + name
			+ "," + isEnabled
			+ ")";
		logger.info(strMsg);
	}

	/**
	 * JBoss generated field access function
	 */
	public abstract void setServerID(Integer serverID);

	/**
	 * JBoss generated field access function
	 */
	public abstract Integer getServerID();

	/**
	 * JBoss generated field access function
	 */
	public abstract void setName(String name);

	/**
	 * JBoss generated field access function
	 */
	public abstract String getName();

	/**
	 * JBoss generated field access function
	 */
	public abstract void setEnable(boolean enable);

	/**
	 * JBoss generated field access function
	 */
	public abstract boolean getEnable();

	/**
	 * Not implemented
	 */
	public void setEntityContext(javax.ejb.EntityContext ctx) {
		this.ctx = ctx;
		logger.info("ServerStatBean setEntityContext()");
	}

	/**
	 * Not implemented
	 */
	public void unsetEntityContext() {
		logger.info("ServerStatBean unsetEntityContext()");
	}

	/**
	 * Not implemented
	 */
	public void ejbActivate() {
		logger.info("ServerStatBean ejbActivate()");
	}

	/**
	 * Not implemented
	 */
	public void ejbPassivate() {
		logger.info("ServerStatBean ejbPassivate()");
	}

	/**
	 * Not implemented
	 */
	public void ejbLoad() {
		logger.info("ServerStatBean ejbLoad()");

	}

	/**
	 * Not implemented
	 */
	public void ejbStore() {
		logger.info("ServerStatBean ejbStore()");

	}

	/**
	 * Not implemented
	 */
	public void ejbRemove() {
		String strMsg = "ServerStatBean ejbRemove(" 
			+ getServerID()
			+ "," + getName()
			+ "," + getEnable()
			+ ")";
		logger.info(strMsg);
	}
}
